/*  Picaso & Diablo 1-Wire Protocol Temporary Implementation

   This include file is a temporary solution to introduce the 1-wire protocol onto Picaso and
   Diablo Display Modules. This is not designed to be for long term use, however should have
   no issues if it was. The 1-wire protocol has been scheduled to be implemented in the core
   of Diablo16 processors, and hopefully also inside Picaso - pending space. This however may
   not occur for some months, so this is an intermediate solution. Enjoy.

   Include file written 13th January 2014 by 4D Systems */

//CONSTANTS for timing of the 1-wire protocol since no microSecond timer exists in 4DGL
#CONST
us60    3    // 3 = 65.2 - 6.8 = 58.4 uS
us75    4    // 4 = 78.6 - 6.8 = 71.8 uS
us400   28   // 28 = 402 - 6.8 = 395.2 uS
us480   34   // 34 = 483 - 6.8 = 476.2 uS
#END

/**********************************************************************
* Function:        OW_Reset()
* Input:           None
* Output:          Return the Presense Pulse from the slave.
* Overview:        Initialization sequence start with reset pulse.
*                  This code generates reset sequence as per the protocol
***********************************************************************/
func OW_Reset()
    var presence_detect;
    pin_Set(OUTPUT, OW_PIN);                // Set OW_PIN as an Output
    pin_HI(OW_PIN);                         // Set OW_PIN Output HI
    pin_LO(OW_PIN);                         // Set OW_PIN Output LO

    delayus(us480);                         // delay 480 microsecond (us)

    pin_Set(INPUT, OW_PIN);                 // Release the bus

    delayus(us75);                          // delay 75 microsecond (us)
    presence_detect := pin_Read(OW_PIN) ;   // Sample for presence pulse from slave

    delayus(us400);                         // delay 400 microsecond (us)

    return presence_detect;
endfunc

/**********************************************************************
* Function:        OW_Write(var write_data)
* Input:           Send byte to 1-wire slave device
* Output:          None
* Overview:        This function used to transmit a complete byte to slave device.
***********************************************************************/
func OW_Write(var write_data)
    var loop ;
    pin_HI(OW_PIN);                          // Set OW_PIN Output HI
    pin_Set(OUTPUT, OW_PIN);                 // Set OW_PIN as Output

    for (loop := 0; loop < 8; loop++)
        if (write_data & 0x01)
            //writing a bit '1'
            pin_LO(OW_PIN);                  // Set OW_PIN Output LO
            pin_HI(OW_PIN);                  // Set OW_PIN Output HI
            delayus(us60);                   // delay 60 microsecond (us)
        else
            //writing a bit '0'
            pin_LO(OW_PIN);                  // Set OW_PIN Output LO
            delayus(us60);                   // delay 60 microsecond (us)
            pin_HI(OW_PIN);                  // Set OW_PIN Output HI
        endif
        write_data := write_data >> 1 ;      // shift the data byte for the next bit to send
    next
endfunc

/**********************************************************************
* Function:        OW_Read()
* Input:           None
* Output:          Return the read byte from slave device
* Overview:        This function used to read a complete byte from the slave device.
***********************************************************************/

func OW_Read()
    var loop, rb;
    var result := 0;
    pin_HI(OW_PIN);                        // Set OW_PIN Output HI
    pin_Set(OUTPUT, OW_PIN);               // Set OW_PIN as Output

    for (loop := 0; loop < 8; loop++)
        result := result >> 1 ;            // shift the result to get it ready for the next bit to receive
        pin_Set(OUTPUT, OW_PIN);           // Set OW_PIN as Output
        pin_LO(OW_PIN);                    // Set OW_PIN Output LO
        pin_HI(OW_PIN);                    // Set OW_PIN Output HI

        pin_Set(INPUT, OW_PIN);            // Set OW_PIN as Input
       //no time for anything here
        rb := pin_Read(OW_PIN);            // Read the status of OW_PIN

        delayus(us60);                     // delay 60 microsecond (us)
        if (rb)
            result |= 0x80;                // if result is one, then set MS-bit
        endif
    next
    return result;
endfunc


//Crude delay for 1-wire Timing created using a simple for loop
func delayus(var val)  //uses constants at top
    var i;
    for (i:=0; i<val; i++);
endfunc
